/*

Copyright (C) 2002-2003, Jeffrey D. Argast.

The authors make NO WARRANTY or representation, either express or implied,
with respect to this software, its quality, accuracy, merchantability, or
fitness for a particular purpose.  This software is provided "AS IS", and you,
its user, assume the entire risk as to its quality and accuracy.

Permission is hereby granted to use, copy, modify, and distribute this
software or portions thereof for any purpose, without fee, subject to these
conditions:

(1) If any part of the source code is distributed, then this
statement must be included, with this copyright and no-warranty notice
unaltered.

(2) Permission for use of this software is granted only if the user accepts
full responsibility for any undesirable consequences; the authors accept
NO LIABILITY for damages of any kind.

*/

#import <Foundation/Foundation.h>
#import <Cocoa/Cocoa.h>

@class PreferenceController;
@class EditSheetController;
@class TextFieldDialogController;
@class EditConfigurationsController;

@interface AppController : NSObject 
{
    IBOutlet id 			fDepotController;
    IBOutlet id				fTabViewDelegate;
    
    PreferenceController*			fPrefController;
	EditSheetController*			fEditSheetController;
	TextFieldDialogController*		fTextFieldDialogController;
	EditConfigurationsController*	fEditConfigurationsController;
	
	IBOutlet NSSplitView*	fHorzSplitView;
	IBOutlet NSSplitView*	fVertSplitView;

	IBOutlet NSMenuItem*	fShowEntireViewMenu;
	IBOutlet NSMenuItem*	fShowClientViewMenu;
	IBOutlet NSMenuItem*	fShowHideDeletedFilesMenu;
	IBOutlet NSMenuItem*	fShowHideLocalFilesMenu;
	
	IBOutlet NSMenu*		fConfigurationMenu;
	IBOutlet NSMenu*		fBookmarksMenu;
	
	IBOutlet id				fBookmarksController;
	IBOutlet NSPanel*		fEditBookmarksPanel;
	NSMutableArray*			fBookmarks;

	NSString*				fClientRoot;
}

- (id) depotController;

- (IBAction) showPreferencePanel: (id) sender;

- (IBAction) editClient: (id) sender;

- (IBAction) submitDefaultChangelist: (id) sender;

// Switch to client view of depot
- (IBAction) clientViewOfDepot: (id) sender;

// switch to entire view of depot
- (IBAction) entireViewOfDepot: (id) sender;

// Show deleted files
- (IBAction) showDeletedFiles: (id) sender;

// Hide deleted files
- (IBAction) hideDeletedFiles: (id) sender;

- (IBAction) editConfigurations: (id) sender;

- (IBAction) setActiveConfiguration: (id) sender;

- (IBAction) editBookmarks: (id) sender;

- (void) addFilesFrom:(NSString*)fromDirectory;
// add files with open panel starting in fromDirectory

- (void) addDepotBookmark:(NSString*)bookmark;

- (void) showEditSheet:(id)sender contentData:(NSString*)contentData endSelector:(SEL)endSelector title:(NSString*)titleString;

@end
