/*

Copyright (C) 2002-2003, Jeffrey D. Argast.

The authors make NO WARRANTY or representation, either express or implied,
with respect to this software, its quality, accuracy, merchantability, or
fitness for a particular purpose.  This software is provided "AS IS", and you,
its user, assume the entire risk as to its quality and accuracy.

Permission is hereby granted to use, copy, modify, and distribute this
software or portions thereof for any purpose, without fee, subject to these
conditions:

(1) If any part of the source code is distributed, then this
statement must be included, with this copyright and no-warranty notice
unaltered.

(2) Permission for use of this software is granted only if the user accepts
full responsibility for any undesirable consequences; the authors accept
NO LIABILITY for damages of any kind.

*/

#import <Foundation/Foundation.h>
#import <Cocoa/Cocoa.h>

@class TextFieldDialogController;
@class PendingChangesController;
@class DepotViewSelection;

@interface DepotController : NSObject 
{
    IBOutlet NSOutlineView*				fOutlineView;
    IBOutlet PendingChangesController*	fPendingChangesController;
	
    NSMutableArray*						fDepots;
    BOOL								fAwake;
    
    TextFieldDialogController*			fTextFieldDialogController;
	
	DepotViewSelection*					fCachedSelection;
}

// this may return nil if there is not a valid item or selection
- (DepotViewSelection*) getSelection;

// called from the app controller after the application has launched and shown the windows
- (void) appFinishedLaunching;

// Something has changed such that we need to throw away the current tree and
// rebuild it
- (void) refreshData;

- (void) expandPath:(NSString*)depotPath;

//
// menu related actions
//

// show the revision history of the selected file
- (IBAction) showRevisionHistory: (id) sender;

// add
- (IBAction) addFile: (id) sender;

// sync
- (IBAction) syncToHead: (id) sender;

// sync
- (IBAction) syncToNone: (id) sender;

// sync
- (IBAction) syncToRevision: (id) sender;

// sync
- (IBAction) syncToChangelist: (id) sender;

// sync
- (IBAction) syncToLabel: (id) sender;

// edit
- (IBAction) edit: (id) sender;

// delete
- (IBAction) delete: (id) sender;

// resolve
- (IBAction) resolve: (id) sender;

// revert
- (IBAction) revert: (id) sender;

// revert -a
- (IBAction) revertUnchanged: (id) sender;

// diff
- (IBAction) diff: (id) sender;

// Open the selection in the editor
- (IBAction) revealInFinder: (id) sender;

// view in editor
- (IBAction) viewInEditor: (id) sender;

// Switch to client view of depot
- (IBAction) clientViewOfDepot: (id) sender;

// switch to entire view of depot
- (IBAction) entireViewOfDepot: (id) sender;

// Show deleted files
- (IBAction) showDeletedFiles: (id) sender;

// Hide deleted files
- (IBAction) hideDeletedFiles: (id) sender;

// add the selection (single selection only) to the bookmarks
- (IBAction) addBookmark: (id) sender;

@end
