/*

Copyright (C) 2002-2003, Jeffrey D. Argast.

The authors make NO WARRANTY or representation, either express or implied,
with respect to this software, its quality, accuracy, merchantability, or
fitness for a particular purpose.  This software is provided "AS IS", and you,
its user, assume the entire risk as to its quality and accuracy.

Permission is hereby granted to use, copy, modify, and distribute this
software or portions thereof for any purpose, without fee, subject to these
conditions:

(1) If any part of the source code is distributed, then this
statement must be included, with this copyright and no-warranty notice
unaltered.

(2) Permission for use of this software is granted only if the user accepts
full responsibility for any undesirable consequences; the authors accept
NO LIABILITY for damages of any kind.

*/

#import "PerforceActionAdd.h"
#import "AppUtils.h"
#import "MessageDefs.h"


@implementation PerforceActionAdd

+ (void) defaultAddFiles:(NSArray*) files;
{
	PerforceActionAdd* action = [[[PerforceActionAdd alloc] init] autorelease];
	
	[action addFiles:files];
}

static NSString* kNewLine = @"\n";

- (void) addFiles:(NSArray*) files
{	
    NSMutableArray* commandAndArgs = [[[NSMutableArray alloc] init] autorelease];
	
	NSMutableString* fileString = [[[NSMutableString alloc] init] autorelease];
	
	int n;
	int num = [files count];
	
	for ( n = 0; n < num; n++ )
	{
		[fileString appendString:[files objectAtIndex:n]];
		
		[fileString appendString:kNewLine];
	}
	
	[commandAndArgs addObject:@"-x"];
	
	[commandAndArgs addObject:@"-"];
	
	[commandAndArgs addObject:@"add"];

    [self runAction:commandAndArgs stdInString:fileString readToEOF:YES];
} 

- (void) dealloc
{
	[fAddedFiles release];

	[super dealloc];
}

static NSString* kAddSep		= @" - ";
static NSString* kOpenForAdd	= @"opened for add";

- (void) processOutput: (NSString*) perforceOutput
{
	NSArray* split = [[perforceOutput substringToIndex: [perforceOutput length] - 1] componentsSeparatedByString: @"\n"];
	NSMutableArray * result = [NSMutableArray arrayWithCapacity: [split count]];
	
	NSString * s;
	NSEnumerator * e = [split objectEnumerator];
	
	while (s = [e nextObject])
	{
		NSArray* addFileArray = [s componentsSeparatedByString:kAddSep];
		
		if ( [addFileArray count] == 2 )
		{
			NSString* addResult = [addFileArray objectAtIndex:1];
			
			if ( [addResult isEqualToString:kOpenForAdd] )
			{
				// Finally, strip off the "#"
				NSArray* finalName = [[addFileArray objectAtIndex:0] componentsSeparatedByString:@"#"];
				[result addObject:[finalName objectAtIndex:0]];
			}
		}
	}
	
	[fAddedFiles release];
	fAddedFiles = [result retain];
	
	[super processOutput:perforceOutput];
}

- (BOOL) isCriticalAction
{
	return YES;
}

- (void) notifyOwnerTaskIsComplete
{
	[super notifyOwnerTaskIsComplete];
	
	if ( [self wasSuccess] )
	{
		SendNotificationWithObject (kActionAddCommandComplete, fAddedFiles);
	}
}

@end
