/*

Copyright (C) 2002-2003, Jeffrey D. Argast.

The authors make NO WARRANTY or representation, either express or implied,
with respect to this software, its quality, accuracy, merchantability, or
fitness for a particular purpose.  This software is provided "AS IS", and you,
its user, assume the entire risk as to its quality and accuracy.

Permission is hereby granted to use, copy, modify, and distribute this
software or portions thereof for any purpose, without fee, subject to these
conditions:

(1) If any part of the source code is distributed, then this
statement must be included, with this copyright and no-warranty notice
unaltered.

(2) Permission for use of this software is granted only if the user accepts
full responsibility for any undesirable consequences; the authors accept
NO LIABILITY for damages of any kind.

*/

#import "PerforceActionSubmit.h"
#import "AppUtils.h"
#import "MessageDefs.h"

static NSString* kErrorTag = @"Error";


@implementation PerforceActionSubmit

+ (void) defaultRunFor:(id)owner selector:(SEL)aSelector form:(NSString*)form
{
	PerforceActionSubmit* action = [[[PerforceActionSubmit alloc] init] autorelease];

	[action runFor:owner selector:aSelector form:form];
}

- (void) dealloc
{
	[fOwner release];
	[fForm release];

	[super dealloc];
}

- (void) runFor:(id)owner selector:(SEL)aSelector form:(NSString*)form
{
    NSMutableArray* commandAndArgs = [[[NSMutableArray alloc] init] autorelease];
	
	fOwner    = [owner retain];
	fSelector = aSelector;
	fForm	  = [form retain];
	
	[commandAndArgs addObject:@"submit"];

	[commandAndArgs addObject:@"-i"];

    [self runAction:commandAndArgs stdInString:form readToEOF:NO sendStdInToConsole:NO];
}

- (NSString*) getForm
{
	return fForm;
}

- (void) processError: (NSString*) perforceError
{
	NSScanner* errScan = [NSScanner scannerWithString:perforceError];
	
	if ( [errScan scanString:kErrorTag intoString:nil] == YES )
	{
		fWasError = YES;
	}
	else
	{
		fWasError = NO;
	}
	
	SendErrorMessage ([perforceError cString]);
}

- (void) notifyOwnerTaskIsComplete
{
	[super notifyOwnerTaskIsComplete];
	
	if ( [self wasSuccess] )
	{
		SendNotification (kActionSubmitCommandComplete);
	}
}

@end


@implementation PerforceActionSubmitChangelist

+ (void) defaultRunFor:(id)owner selector:(SEL)aSelector changeNum:(NSString*)changeNum
{
	PerforceActionSubmitChangelist* action = [[[PerforceActionSubmitChangelist alloc] init] autorelease];

	[action runFor:owner selector:aSelector changeNum:changeNum];
}

- (void) dealloc
{
	[fOwner release];
	[fChangeNum release];

	[super dealloc];
}

- (void) runFor:(id)owner selector:(SEL)aSelector changeNum:(NSString*)changeNum
{
    NSMutableArray* commandAndArgs = [[[NSMutableArray alloc] init] autorelease];
	
	fOwner    	= [owner retain];
	fSelector 	= aSelector;
	fChangeNum  = [changeNum retain];
	
	[commandAndArgs addObject:@"submit"];

	[commandAndArgs addObject:@"-c"];

	[commandAndArgs addObject:changeNum];

    [self runAction:commandAndArgs stdInString:nil readToEOF:NO];
}

- (NSString*) getChangeNum
{
	return fChangeNum;
}

- (void) processError: (NSString*) perforceError
{
	NSScanner* errScan = [NSScanner scannerWithString:perforceError];
	
	if ( [errScan scanString:kErrorTag intoString:nil] == YES )
	{
		fWasError = YES;
	}
	else
	{
		fWasError = NO;
	}
	
	SendErrorMessage ([perforceError cString]);
}

- (void) notifyOwnerTaskIsComplete
{
	[super notifyOwnerTaskIsComplete];
	
	if ( [self wasSuccess] )
	{
		SendNotification (kActionSubmitCommandComplete);
	}
}

@end
