/*

Copyright (C) 2002-2003, Jeffrey D. Argast.

The authors make NO WARRANTY or representation, either express or implied,
with respect to this software, its quality, accuracy, merchantability, or
fitness for a particular purpose.  This software is provided "AS IS", and you,
its user, assume the entire risk as to its quality and accuracy.

Permission is hereby granted to use, copy, modify, and distribute this
software or portions thereof for any purpose, without fee, subject to these
conditions:

(1) If any part of the source code is distributed, then this
statement must be included, with this copyright and no-warranty notice
unaltered.

(2) Permission for use of this software is granted only if the user accepts
full responsibility for any undesirable consequences; the authors accept
NO LIABILITY for damages of any kind.

*/

#import "PerforceActionSync.h"
#import "AppUtils.h"
#import "MessageDefs.h"

static NSString* kNewLine = @"\n";

@implementation PerforceActionSync

+ (void) defaultRun:(NSArray*)depotPaths revision:(NSString*)revision
{
	PerforceActionSync* action = [[[PerforceActionSync alloc] init] autorelease];
	
	[action run:depotPaths revision:revision];
}

- (void) run:(NSArray*)depotPaths revision:(NSString*)revision
{
	if ( depotPaths )
	{
		fGlobal = NO;
		fPathArray = [depotPaths retain];
		
		NSMutableArray* commandAndArgs = [[[NSMutableArray alloc] init] autorelease];
		
		NSMutableString* fileString = [[[NSMutableString alloc] init] autorelease];
		
		int n;
		int num = [depotPaths count];
		
		for ( n = 0; n < num; n++ )
		{
			[fileString appendString:[depotPaths objectAtIndex:n]];
			
			if ( revision )
			{
				[fileString appendString:revision];
			}
			
			[fileString appendString:kNewLine];
		}
		
		[commandAndArgs addObject:@"-x"];
		
		[commandAndArgs addObject:@"-"];
		
		[commandAndArgs addObject:@"sync"];
	
		[self runAction:commandAndArgs stdInString:fileString readToEOF:NO];
	}
	else
	{
		fGlobal = YES;

		NSMutableArray* commandAndArgs = [[[NSMutableArray alloc] init] autorelease];
				
		[commandAndArgs addObject:@"sync"];
		
		if ( revision )
		{
			[commandAndArgs addObject:revision];
		}
		
		[self runAction:commandAndArgs stdInString:nil readToEOF:NO];
	}
}

- (void) dealloc
{
	[fOwner release];
	[fPathArray release];
	
	[super dealloc];
}

- (BOOL) isCriticalAction
{
	return YES;
}

- (void) notifyOwnerTaskIsComplete
{
	[super notifyOwnerTaskIsComplete];
	
	if ( [self wasSuccess] )
	{
		if ( fGlobal == NO )
		{
			SendNotificationWithObject (kActionSyncCommandComplete, fPathArray);
		}
		else
		{
			SendNotification (kGlobalSyncComplete);
		}
	}
}

@end
