/*

Copyright (C) 2002-2003, Jeffrey D. Argast.

The authors make NO WARRANTY or representation, either express or implied,
with respect to this software, its quality, accuracy, merchantability, or
fitness for a particular purpose.  This software is provided "AS IS", and you,
its user, assume the entire risk as to its quality and accuracy.

Permission is hereby granted to use, copy, modify, and distribute this
software or portions thereof for any purpose, without fee, subject to these
conditions:

(1) If any part of the source code is distributed, then this
statement must be included, with this copyright and no-warranty notice
unaltered.

(2) Permission for use of this software is granted only if the user accepts
full responsibility for any undesirable consequences; the authors accept
NO LIABILITY for damages of any kind.

*/

#import <Cocoa/Cocoa.h>

#import "PerforceDepot.h"
#import "PerforceAction.h"
#import "IconController.h"
#import "IconDefs.h"
#import "MessageDefs.h"
#import "PerforceFile.h"
#import "AppUtils.h"

@interface PerforceDepot(Private)

- (id) initWithString: (NSString *) string;
- (id) initWithPath: (NSString *) string depotType: (int) type;

- (void) setDepotType: (int) type;

@end


@implementation PerforceDepot

+(PerforceDepot *) depotWithString: (NSString *) string
{
	return [[[PerforceDepot alloc] initWithString: string] autorelease];
}

+(PerforceDepot *) depotWithPath: (NSString *) string depotType: (int) type
{
	return [[[PerforceDepot alloc] initWithPath: string depotType: type] autorelease];
}

// One line of output from "p4 depots" command
- (id) initWithString: (NSString *) string;
{
	NSScanner * scanner = [NSScanner scannerWithString: string];
	NSString * path = nil;
	
	[scanner scanString: @"Depot" intoString: nil] &&
	[scanner scanUpToString: @" " intoString: &path] &&
	[scanner scanUpToString: @" " intoString: nil];		// date
	if ([scanner scanString: @"local" intoString: nil])
	{
		return [self initWithPath: path depotType: kLocalDepotType];
	}
	else
	{
		return [self initWithPath: path depotType: kRemoteDepotType];
	}
}

// A path and type, from "p4 dirs -C //*" command
- (id) initWithPath: (NSString *) string depotType: (int) type
{
	if (self = [super initWithPath: [NSString stringWithFormat: @"//%@", string]])
	{
		fType = type;
		
		IconController* iconController = [IconController defaultIconController];

		// release because it was retained in base class
		[fIcon release];

		if (fType == kLocalDepotType)
			fIcon = [[iconController getIcon:kLocalDepotIcon] retain];
		else
			fIcon = [[iconController getIcon:kRemoteDepotIcon] retain];
	}
    return self;
}

- (void) dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	[super dealloc];
}

- (void) setDepotType: (int) type
{
	fType = type;
	
	IconController* iconController = [IconController defaultIconController];

	// release because it was retained in base class
	[fIcon release];

	if (fType == kLocalDepotType)
		fIcon = [[iconController getIcon:kLocalDepotIcon] retain];
	else
		fIcon = [[iconController getIcon:kRemoteDepotIcon] retain];
}



- (NSComparisonResult)compare:(PerforceDepot *)aDepot
{
	return [fDepotPath compare: aDepot->fDepotPath];
}

- (id) getCellText
{
    return fDepotPath;
}

@end
